/* Copyright (C) 1998 Oregon Graduate Institute of Science & Technology
 * Portions copyright (c) 1995 Brown University.
 * Copyright (c) 1995 The Regents of the University of California.
 * 
 * See the file COPYRIGHT, which should have been supplied with this
 * package, for licensing details.  We may be contacted through email
 * at <quasar-help@cse.ogi.edu>.
 */

/*
 * main.c --
 *
 *      Main procedure
 *
 */

#include "global.h"
#include "video.h"
#include "util.h"
#include "gdith.h"
#include "stream_play.h"

#include <string.h>
#include <signal.h>
#include <errno.h>
#include <dbug.h>

#ifdef HAVE_LIBSWQ
#include <swqueue.h>
#include <smba.h>
#endif


/* Define buffer length. */
#define BUF_LENGTH 4096


static void
do_cleanup(int sig) {
  DBUG_ENTER();

  fprintf(stderr, "Caught signal %d.  Exiting...\n", sig);
  exit(1);
  DBUG_VOID_RETURN();
}



/*** is_uint
 *
 *	Ensure we have an unsigned int.
 *
**/

static int
is_uint(char *val)
{
  char *c = val;
  DBUG_ENTER();

  if (!*c)
    DBUG_RETURN(0);
  for (; *c; ++c) {
    if (!isdigit(*c))
      DBUG_RETURN(0);
  }
  DBUG_RETURN(1);
}



/*** usage --
 *
 *	Print usage statement and quit.
 *
**/

static volatile void
usage(char *prog) {
  DBUG_ENTER();
  fprintf(stderr, "usage: %s "
#ifdef HAVE_LIBSWQ
#ifdef HAVE_LIBPTHREAD
      "[-X] "
#endif
      "{ -Q queue-id smba-id [filename] | filename }\n",
#else
      "filename\n",
#endif
      prog);
  exit(1);
  DBUG_VOID_RETURN();
}



/*
 *--------------------------------------------------------------
 *
 * main --
 *
 *        Parses command line, starts decoding and displaying.
 *
 *--------------------------------------------------------------
 */

#ifndef __STDC__
void
#endif
main(int argc, char **argv)
{
  char *prog = basename(*argv);
  char *inputName;
  FILE *input = 0;
  DataStream *theSource = NULL;
  VidStream *theStream = NULL;
  struct sigaction cleanup;
#ifdef HAVE_LIBSWQ
  int use_queue = 0, report_queue = 0;
#endif
  FILE *log;
  int ditherType;
  DBUG_ENTER();
  DBUG_KEYWORDS("display",NULL);

  g_thread_init(NULL);
  gtk_init(&argc, &argv);
  /* Fix up signals. */

  cleanup.sa_handler = do_cleanup;
  cleanup.sa_flags = 0;
  cleanup.sa_restorer = 0;
  sigemptyset(&cleanup.sa_mask);
  sigaddset(&cleanup.sa_mask, SIGHUP);
  sigaddset(&cleanup.sa_mask, SIGINT);
  sigaddset(&cleanup.sa_mask, SIGPIPE);
  sigaddset(&cleanup.sa_mask, SIGALRM);
  sigaddset(&cleanup.sa_mask, SIGTERM);
  
  /* NB: Go on even if errors occur. */
  sigaction(SIGHUP, &cleanup, 0);
  sigaction(SIGINT, &cleanup, 0);
  sigaction(SIGPIPE, &cleanup, 0);
  sigaction(SIGALRM, &cleanup, 0);
  sigaction(SIGTERM, &cleanup, 0);


  /* Set up tables. */
  InitEverything();

  /* Parse args. */
  for (;;) {
    int c = getopt(argc, argv, ""
#ifdef HAVE_LIBSWQ
#ifdef HAVE_LIBPTHREAD
	"X"
#endif
	"Q"
#endif
	);

    switch (c) {
#ifdef HAVE_LIBSWQ
#ifdef HAVE_LIBPTHREAD
     case 'X':
      report_queue = 1;
      break;
#endif

     case 'Q':
      use_queue = 1;
      break;
#endif

     default:
      usage(prog);	/* DOES NOT RETURN */

     case EOF:
      goto opts_done;
    }
  }
opts_done:

  /* Create data stream. */
  theSource = NewDataStream((unsigned int) BUF_LENGTH);

#ifdef HAVE_LIBSWQ
  if (use_queue) {	/* Open queue. */
    swqRegion region;
    char *term;
    int qnum, bnum;

    term = argv[optind++];
    if (!term || !is_uint(term))
      usage(prog);

    qnum = atoi(term);

    term = argv[optind++];
    if (!term || !is_uint(term))
      usage(prog);

    bnum = atoi(term);


    region.sr_addr.sa_tag = swq_sysvshmem;
    region.sr_addr.u.sysv.sa_key = qnum;
    theSource->queue = swqOpen(&region, SWQ_CONSUMER | 0600);
    if (!theSource->queue) {
      fprintf(stderr, "%s: swqOpen failed (%s)\n", prog, strerror(errno));
      exit(1);
    }

    theSource->buf_area = smbaopen(bnum, 0, 0, SMBA_VSIZE | 0600);
    if (!theSource->buf_area) {
      fprintf(stderr, "%s: smbaopen failed (%s)\n", prog, strerror(errno));
      exit(1);	/* XXX Clean up queue. */
    }


    inputName = argv[optind++];
    if (inputName && argv[optind])
      usage(prog);
    
    if (!inputName)
      inputName = "-";
  }
  else {		/* Open file. */
#endif /* HAVE_LIBSWQ */
    inputName = argv[optind++];
    if (!inputName || argv[optind])
      usage(prog);

    if (!strcmp(inputName, "-"))
      input = stdin;
    else
      input = fopen(inputName, "rb");

    if (!input) {
      /* Flagrantly assume stdin != NULL */
      fprintf(stderr, "%s: %s: fopen to read failed (%s)\n", prog, inputName,
	  strerror(errno));
      exit(1);
    }

    theSource->input = input;
#ifdef HAVE_LIBSWQ
  }
#endif
  ditherType = InitDithering();
  theStream = NewVidStream(theSource, inputName, ditherType);
#ifdef HAVE_LIBSWQ
#ifdef HAVE_LIBPTHREAD
  if (report_queue) {
    printf("%d\n", theStream->queue_region.sr_addr.u.sysv.sa_key);
    fflush(stdout);
  }
#endif
#endif

  theStream->rate_ticks = -1;	/* Activate frame rate control. */
  if (!mpegVidRsrc(theStream)) {
    fprintf(stderr, "Skipping movie \"%s\" - not an MPEG stream\n", inputName);
    goto finishUp;
  }

  while (!theStream->film_has_ended) {
    mpegVidRsrc(theStream);
  }
#if 0
  if (loopFlag) {
    ResetVidStream(theStream); /* Reinitialize VidStream pointers */
    mpegVidRsrc(theStream); /* Process start codes */
  }
#endif

finishUp:
  DestroyVidStream(theStream);
  DestroyDataStream(theSource);

  if (input)
    fclose(input);

  exit(0);
  DBUG_VOID_RETURN();
}
 
